/*************************************************************************
 *
 *    (c) Olimex Ltd. 2010
 *
 *    File name   : BSP.h
 *    Description : Board support package
 *
 *    History :
 *    1. Date        : 25 May 2010
 *       Author      : Aleksandar Mitev
 *       Description : Create
 *
 **************************************************************************/

#ifndef _BSP_H
#define _BSP_H

#define LOOP_DLY_100US        450

/*************************************************************************
 * Function Name: InitClock
 * Parameters: none
 *
 * Return: none
 *
 * Description: Init MCU clock
 *
 *************************************************************************/
void InitClock (void);

/*************************************************************************
 * Function Name: InitUserPeriph
 * Parameters: none
 *
 * Return: none
 *
 * Description: initializes pins used by the LEDS, USB connect input, wake-up button input
 *
 *************************************************************************/
void InitUserPeriph(void);

/*************************************************************************
 * Function Name: StatLEDSet
 * Parameters: ledno - index of the led to alter
 *			enabled - TRUE to turn on, FALSE to turn off
 *
 * Return: none
 *
 * Description: turns on/off LED
 *
 *************************************************************************/
void StatLEDSet(int ledno, Boolean enabled);

/*************************************************************************
 * Function Name: USBConnectedPinGet
 * Parameters: none
 *
 * Return: TRUE if cable is connected, FALSE if not
 *
 * Description: Returns a flag indicating whether a USB cable is connected
 *
 *************************************************************************/
Boolean USBConnectedPinGet(void);

/*************************************************************************
 * Function Name: PowerJackConnectedPinGet
 * Parameters: none
 *
 * Return: TRUE if power cable jack is connected, FALSE if not
 *
 * Description: Returns a flag indicating whether a power jack cable is connected
 *
 *************************************************************************/
Boolean PowerJackConnectedPinGet(void);

/*************************************************************************
 * Function Name: USBConnectedPinGet
 * Parameters: none
 *
 * Return: TRUE if cable is pressed, FALSE if not
 *
 * Description: Returns a flag indicating whether a USB cable is connected
 *
 *************************************************************************/
Boolean WakeupButtonGet(Boolean relaxEnable);

/*******************************************************************************
* Function Name  : ResetDevice
* Description    : Software reset
* Input          : Node
* Output         : None
* Return         : There is no return here!
*******************************************************************************/
void ResetDevice(void);

/*************************************************************************
 * Function Name: Dly100us
 * Parameters: void *arg
 *
 * Return: none
 *
 * Description: Delay 100us * arg
 *
 *************************************************************************/
void Dly100us(void *arg);

#endif // _BSP_H
