/*************************************************************************
 *
 *    (c) Olimex Ltd. 2010
 *
 *    File name   : BattMeas.h
 *    Description : Functions for measuring battery voltage.
 *
 *    Synopsis: This is nonblocking implementation of FSM for measurnig voltage with the ADC module.
 *              Since Vdda is not stable if battery voltage drops below 3.3V, INTREF (channel 17) (which is 1.2V) is
 *              used as reference. The formula used to calculate battery voltage is:
 *                    Vbatt = (battMeasuredSteps * 1.2) / refMeasuredSteps
 *
 *    History :
 *    1. Date        : 25 May 2010
 *       Author      : Aleksandar Mitev
 *       Description : Create
 *
 **************************************************************************/

#ifndef _BATT_MEAS_H
#define _BATT_MEAS_H

/*************************************************************************
 * Function Name: BattMeasInit
 * Parameters: none
 *
 * Return: none
 *
 * Description: Initialize ADC channel 10 and enable pin
 *
 *************************************************************************/
void BattMeasInit(void);


/*************************************************************************
 * Function Name: BattMeasDoWork(..)
 * Parameters: none
 *
 * Return: none
 *
 * Description:
 *
 *************************************************************************/
void BattMeasDoWork(void);

/*************************************************************************
 * Function Name: BattMeasGetVoltage(..)
 * Parameters: none
 *
 * Return: none
 *
 * Description:
 *
 *************************************************************************/
float BattMeasGetVoltage(void);

#endif // _BATT_MEAS_H