/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2006
 *
 *    File name   : main.c
 *    Description : main module
 *
 *    History :
 *    1. Date        : October 10, 2006
 *       Author      : Stanimir Bonev
 *       Description : Create
 *
 *  This example project shows how to use the IAR Embedded Workbench for ARM
 * to develop code for the IAR STM32-SK evaluation boards and MOD-MP3 module.
 *  It implements a MP3 player. The MOD-MP3 module must be connected to the
 * UEXT connector of the IAR STM32-SK board. The player can read MMC/SD media
 * formatting under FAT12/16/32 file system.
 *
 *  Jumpers:
 * 	PWR_SEL - depending of power source
 *
 *  Buttons:
 *   	B1 - Next a file/dir (select mode)
 *   	B1 - Sound effects None, Wide Stereo, Loudness (play mode)
 *   	B2 - Enter/exit form directory or Play/Stop
 *  Potentiometers:
 *   	AN_TR - volume control
 *
 *    $Revision: 31601 $
 *
 **************************************************************************/
#include    "includes.h"

// declarations of the tasks
void MassStorageTask(void);
void Mp3PlayerTask(void);

Int32U CriticalSecCntr;

/*************************************************************************
 * Function Name: main
 * Parameters: none
 *
 * Return: none
 *
 * Description:
 *
 *************************************************************************/
int main (void)
{

#ifdef DEBUG
 debug();
#endif

  ENTR_CRT_SECTION();

  // Init clock system
  InitClock();

  // NVIC init
#ifndef  EMB_FLASH
  // Set the Vector Table base location at 0x20000000
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0);
#else  // VECT_TAB_FLASH
  // Set the Vector Table base location at 0x08000000
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);
#endif
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_4);

  // Enable the Interrupt controller to manage IRQ channel
  __enable_interrupt();

  EXT_CRT_SECTION();

	// Init leds, usb connect and wakeup button
	InitUserPeriph();
	
	Dly100us((void*)10);
	
	if( USBConnectedPinGet() )
		MassStorageTask();
	else
		Mp3PlayerTask();
	
	while(1);
}

#ifdef  DEBUG
/*******************************************************************************
* Function Name  : assert_failed
* Description    : Reports the name of the source file and the source line number
*                  where the assert_param error has occurred.
* Input          : - file: pointer to the source file name
*                  - line: assert_param error line source number
* Output         : None
* Return         : None
*******************************************************************************/
void assert_failed(u8* file, u32 line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

